package com.uas.ps.message;

import com.uas.ps.message.dao.CronDao;
import com.uas.ps.message.domain.Cron;
import com.uas.ps.message.exception.ParameterMissingException;
import com.uas.ps.message.service.MessageService;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.ScheduledFuture;
import javax.annotation.Resource;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.Trigger;
import org.springframework.scheduling.TriggerContext;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.util.StringUtils;

/**
 * Created by wangyc on 2018/1/31.
 *
 * @version 2018/1/31 19:16 wangyc
 */
@Configuration
@EnableScheduling
public class SchedulingConfig implements SchedulingConfigurer {

    private final Logger logger = Logger.getLogger(Logger.class);

    private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("HH:mm:ss");

    private String cornStr = "";

    @Resource
    private MessageService messageService;

    @Resource
    private CronDao cronDao;

    @Override
    public void configureTasks(ScheduledTaskRegistrar scheduledTaskRegistrar) {
        scheduledTaskRegistrar.addTriggerTask(new Runnable() {
            @Override
            public void run() {
            messageService.sendMessage("{\"consumerApp\":\"B2B\"}");
            messageService.sendMessage("{\"consumerApp\":\"HIDE\"}");
            // 定时任务的业务逻辑
            logger.info("动态修改定时任务cron参数，当前时间：" + DATE_FORMAT.format(new Date()));
            }
        }, new Trigger() {
            @Override
            public Date nextExecutionTime(TriggerContext triggerContext) {
                // 定时任务触发，可修改定时任务的执行周期
                final Cron cron = cronDao.findOne(1L);
                if (cron != null && !StringUtils.isEmpty(cron.getCron())) {
                    cornStr = cron.getCron();
                }

                CronTrigger trigger = new CronTrigger(cron.getCron());
                Date nextExecDate = trigger.nextExecutionTime(triggerContext);
                logger.info("下次执行自动推送时间： " + DATE_FORMAT.format(nextExecDate));
                return nextExecDate;
            }
        });
    }

    /**
     * 设置定时频率
     * @param cron 定时频率
     * @return
     */
    public String setCron(String cron) {
        if (!StringUtils.isEmpty(cron)) {
            this.cornStr = cron;
        } else {
            throw new ParameterMissingException("定时频率为空");
        }
        return this.cornStr;
    }
}
